/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2011  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#include "base.h"

#include <cerrno>
#include <cstring>

#if IVALUE==IINT
#include <climits>
#endif

using namespace std;

#ifndef HAVE_EXP10_SET
ieee exp10(ieee x){
  return exp(LOG10*x);
}
#endif

value::value(){
  frac_ready=1;
  type=INTEGER;
}

value::value(itype ii){
  frac_ready=1;
  type=INTEGER;
  i=ii;
  d=ii;
  f=0;
}

void value::fractionalize(){
  if(frac_ready)return;
  frac_ready=1;
  if(d>=0){
    i=(itype)d;
    f=d-i;
    return;
  }
  i=(itype)(d-(itype)d+1)+(itype)d-1;
  f=d-i;
}

value::value(ieee ff){
  frac_ready=0;
  type=FLOATING;
  d=ff;
  //fractionalize(ff,i,f);
}

value::value(const char* s){
  if(strchr(s,'.')||strchr(s,'e')||strchr(s,'E')){
    frac_ready=0;
    type=FLOATING;
#if VALUE==LONG
    d=strtold(s,0);
#elif VALUE==DOUBLE
    d=strtod(s,0);
#else
    d=strtof(s,0);
#endif
    return;
  }else{
    frac_ready=1;
    type=INTEGER;
#if IVALUE==ILONG
    //    return value((itype)strtoll(val,0,10));
    errno=0;
    long long int ival=strtoll(s,0,10);
    if(errno){
      fprintf(stderr,"parse: integer overflow during parsing input\n");
      type=FLOATING;
      d=(0./0.);
      i=-1;
    }
    else{
      i=ival;
      d=i;
    }

#else
    //    return value((itype)strtol(val,0,10));
    errno=0;
    long int ival=strtol(s,0,10);
    if(errno || ival>INT_MAX || ival<INT_MIN){
      fprintf(stderr,"parse: integer overflow during parsing input\n");
      type=FLOATING;
      d=(0./0.);
      i=-1;
    }else{
      i=ival;
      d=i;
    }

#endif
    f=0;
    return;
  }
}

itype value::assertI(const char* op){
  if(type!=INTEGER){
    fprintf(stderr,"warning: integer required by '%s', converting\n",op);
    fractionalize();
  }
  return i;
}

ieee value::assertD(const char* op){
  if(type!=FLOATING){
    fprintf(stderr,"warning: floating required by '%s', converting\n",op);
  }
  return d;
}

value value::operator+(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){

#ifdef TRAP_OVR
    char flag;
    itype result=i;
    itype operand=v.i;
#if IVALUE==IINT
    __asm__ __volatile__ (
			"addl %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#elif IVALUE==ILONG
    __asm__ __volatile__ (
			"addq %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#endif
    if(flag){
#if DEBUG>0
      fprintf(stderr,"runtime: integer overflow on addition, casting to float\n");
#endif
      return value(d+v.d);
    }else{
      return value(result);
    }

#else
    return value(i+v.i);
#endif
  }else{
    return value(d+v.d);
  }
}

value value::operator-(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){

#ifdef TRAP_OVR
    char flag;
    itype result=i;
    itype operand=v.i;
#if IVALUE==IINT
    __asm__ __volatile__ (
			"subl %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#elif IVALUE==ILONG
    __asm__ __volatile__ (
			"subq %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#endif
    if(flag){
#if DEBUG>0
      fprintf(stderr,"runtime: integer overflow on subtraction, casting to float\n");
#endif
      return value(d-v.d);
    }else{
      return value(result);
    }

#else
    return value(i-v.i);
#endif
  }else{
    return value(d-v.d);
  }
}

value value::operator*(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){

#ifdef TRAP_OVR
    char flag;
    itype result=i;
    itype operand=v.i;
#if IVALUE==IINT
    __asm__ __volatile__ (
			"imull %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#elif IVALUE==ILONG
    __asm__ __volatile__ (
			"imulq %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#endif
    if(flag){
#if DEBUG>0
      fprintf(stderr,"runtime: integer overflow on multiplication, casting to float\n");
#endif
      return value(d*v.d);
    }else{
      return value(result);
    }

#else
    return value(i*v.i);
#endif
  }else{
    return value(d*v.d);
  }
}

inline static itype imul(itype a, itype b, char& flag){
#ifdef TRAP_OVR
    char fl;
#if IVALUE==IINT
    __asm__ __volatile__ (
			"imull %2,%0\n\t"
			"seto %1\n"
			:"=g"(a), "=ir"(fl)
			:"ir"(b), "0"(a)
			);
#elif IVALUE==ILONG
    __asm__ __volatile__ (
			"imulq %2,%0\n\t"
			"seto %1\n"
			:"=g"(a), "=ir"(fl)
			:"ir"(b), "0"(a)
			);
#endif
    flag|=fl;

#if DEBUG>0
    if(flag){
      fprintf(stderr,"runtime: integer overflow on multiplication\n");
    }
#endif
    return a;
#else
    return a*b;
#endif
}


value value::operator/(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    if(v.i==0)return value((itype)0);
    return value(i/v.i);
  }else{
    if(v.d==0)return value((itype)0);
    return value(d/v.d);
  }
}

static value flpow(ieee i1, itype i2){
  if(i2<0){
    return value(pow(i1,(int)i2));
  }else{
    int sign=(i1<0) && (i2&1);
    i1=i1<0?-i1:i1;
    ieee trailing=i1;
    ieee total=1;
    while(i2!=0){
      if(i2&1)total*=trailing;
      trailing*=trailing;
      i2>>=1;
    }
    if(sign)total=-total;
    return value(total);
  }
}

static value intpow(itype i1, itype i2){
  if(i2<0){
    return value(pow((ieee)i1,(int)i2));
  }else{
    char flag=0;
    int sign=(i1<0) && (i2&1);
    itype base=i1<0?-i1:i1;
    itype trailing=base;
    itype total=1;
    itype ex=i2;
    while(ex!=0){
      if(ex&1)total=imul(total,trailing,flag);
      trailing=imul(trailing,trailing,flag);
      ex>>=1;
    }
    if(flag)return flpow((ieee)i1,i2);
    if(sign)total=-total;
    return value((itype)total);
  }
}

value value::operator^(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    return intpow(i,v.i);
  }else if(v.type==INTEGER){
    return flpow(d,v.i);
  }else{
    return value(pow(d,v.d));
  }
}

static ieee d_mod(ieee d1, ieee d2){
  ieee r=d1-(floor(d1/d2))*d2;
  return r<0?0:r;
}

static ieee f_mod(ieee v1, ieee v2){
  if(v2==0)return v1;
  if(v2>=0){
    return d_mod(v1,v2);
  }else{
    v2=-v2;
    return d_mod(v1,v2)-v2;
  }
}

static itype i_mod(itype i1, itype i2){
  if(i2==0)return i1;
  if(i2>=0){
    return i1<0?(i2-(-i1)%i2)%i2:i1%i2;
  }else{
    i2=-i2;
    return (i1<0?(i2-(-i1)%i2)%i2:i1%i2)-i2;
  }
}

value value::operator%(const value& v){
  if((type==INTEGER)&&(v.type==INTEGER)){
    return value(i_mod(i,v.i));
  }else{
    return value(f_mod(d,v.d));
  }
}



value value::operator+(const ieee& v){
  return value(d+v);
}

value value::operator-(const ieee& v){
  return value(d-v);
}

value value::operator*(const ieee& v){
  return value(d*v);
}

value value::operator/(const ieee& v){
  if(v==0)return value((itype)0);
  return value(d/v);
}

value value::operator^(const ieee& v){
  return value(pow(d,v));
}

value value::operator%(const ieee& v){
  return value(f_mod(d,v));
}


value value::operator+(const itype& v){
  if(type==INTEGER){
#ifdef TRAP_OVR
    char flag;
    itype result=i;
    itype operand=v;
#if IVALUE==IINT
    __asm__ __volatile__ (
			"addl %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#elif IVALUE==ILONG
    __asm__ __volatile__ (
			"addq %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#endif
    if(flag){
#if DEBUG>0
      fprintf(stderr,"runtime: integer overflow on addition, casting to float\n");
#endif
      return value(d+(ieee)v);
    }else{
      return value(result);
    }

#else
    return value(i+v);
#endif
  }else{
    return value(d+v);
  }
}

value value::operator-(const itype& v){
  if(type==INTEGER){
#ifdef TRAP_OVR
    char flag;
    itype result=i;
    itype operand=v;
#if IVALUE==IINT
    __asm__ __volatile__ (
			"subl %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#elif IVALUE==ILONG
    __asm__ __volatile__ (
			"subq %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#endif
    if(flag){
#if DEBUG>0
      fprintf(stderr,"runtime: integer overflow on subtraction, casting to float\n");
#endif
      return value(d-(ieee)v);
    }else{
      return value(result);
    }

#else
    return value(i-v);
#endif
  }else{
    return value(d-v);
  }
}

value value::operator*(const itype& v){
  if(type==INTEGER){
#ifdef TRAP_OVR
    char flag;
    itype result=i;
    itype operand=v;
#if IVALUE==IINT
    __asm__ __volatile__ (
			"imull %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#elif IVALUE==ILONG
    __asm__ __volatile__ (
			"imulq %2,%0\n\t"
			"seto %1\n"
			:"=g"(result), "=ir"(flag)
			:"ir"(operand), "0"(result)
			);
#endif
    if(flag){
#if DEBUG>0
      fprintf(stderr,"runtime: integer overflow on multiplication, casting to float\n");
#endif
      return value(d*(ieee)v);
    }else{
      return value(result);
    }

#else
    return value(i*v);
#endif
  }else{
    return value(d*v);
  }
}

value value::operator/(const itype& v){
  if(v==0)return value((itype)0);
  if(type==INTEGER){
    return value(i/v);
  }else{
    return value(d/v);
  }
}

value value::operator^(const itype& v){
  if(type==INTEGER){
    return intpow(i,v);
  }else {
    return flpow(d,v);
  }
}

value value::operator%(const itype& v){
  if(type==INTEGER){
    return value(i_mod(i,v));
  }else{
    return value(f_mod(d,v));
  }
}


value value::operator-(){
  if(type==INTEGER){
    return value(-i);
  }else{
    return value(-d);
  }
}

value value::op_sin(){
  return value(sin(d));
}

value value::op_cos(){
  return value(cos(d));
}

value value::op_tan(){
  return value(tan(d));
}

value value::op_atan(){
  return value(atan(d));
}

value value::op_asin(){
  return value(asin(d));
}

value value::op_acos(){
  return value(acos(d));
}

value value::op_abs(){
  if(type==INTEGER){
    return value(i<0?-i:i);
  }else{
    return value(fabs(d));
  }
}

value value::op_exp(){
  return value(exp(d));
}

value value::op_sqrt(){
  return value(sqrt(d));
}

value value::op_cbrt(){
  return value(cbrt(d));
}

value value::op_log(){
  return value(log(d));
}


value value::op_bj(int n){
  return value(jn(n,d));
}

value value::op_by(int n){
  return value(yn(n,d));
}

value value::cast_int(){
  fractionalize();//ensure i and f are available
  return value(i);
}

value value::cast_float(){
  return value(d);
}

value value::op_frac(){
  fractionalize();//ensure i and f are available
  return value(f);
}

value value::op_round(){
  fractionalize();//ensure i and f are available
  return value(f>=0.5?i+1:i);
}

#if IVALUE==ILONG
#define MAX_FACT_INT 20
#else
#define MAX_FACT_INT 12
#endif

value value::op_fact(){
  if(type==INTEGER){
    if(i>MAX_FACT_INT){
      ieee fac=1;
      for(itype u=2;u<=i;u++)fac*=u;
      return value(fac);
    }else{
      itype fac=1;
      for(itype u=2;u<=i;u++)fac*=u;
      return value(fac);
    }
  }else{
    int gi;
    ieee lg=lgamma_r(d+1,&gi);
    return value(gi*exp(lg));
  }
}

value value::op_exp10(){
  return value(exp10(d));
}

value value::op_rand(){
  if(type==INTEGER){
    if(!i)return value((itype)0);
    int sign=i<0?-1:1;
    return value((itype)(sign*(rand()%(sign*i))));
  }else{
    return value((d*rand())/RAND_MAX);
  }
}

value value::binomial(value v1, value v2){
  if(v2.type==INTEGER){
    value total=(itype)1;
    value nom=v1;
    for(int i=1;i<=v2.i;i++){
      total=total*v1;
      total=total/(itype)i;
      v1=v1-(itype)1;
    }
    return total;
  }else{
    int i1, i2, i3;
    ieee g1=lgamma_r(v1.d+1,&i1);
    ieee g2=lgamma_r(v2.d+1,&i2);
    ieee g3=lgamma_r(v1.d-v2.d+1,&i3);
    return value((ieee)((i1*i2*i3)*exp(g1-g2-g3)));
  }
}

int value::is_odd(){
  if(type!=INTEGER)return 0;
  if(i<0){
    return (-i)%2==1;
  }else{
    return i%2==1;
  }
}

int value::is_even(){
  if(type!=INTEGER)return 0;
  if(i<0){
    return (-i)%2==0;
  }else{
    return i%2==0;
  }
}

int value::is_positive(){
  if(type==INTEGER)return i>=0;
  else return d>=0;
}

int value::is_negative(){
  if(type==INTEGER)return i<0;
  else return d<0;
}

int value::is_int(){
  return type==INTEGER;
}

int value::is_float(){
  return type==FLOATING;
}

int value::operator<(const value& v){
  return d<v.d;
}

int value::operator>(const value& v){
  return d>v.d;
}

int value::operator<=(const value& v){
  return d<=v.d;
}

int value::operator>=(const value& v){
  return d>=v.d;
}

int value::operator==(const value& v){
  if(type!=v.type)return 0;
  if(type==INTEGER)return i==v.i;
  else return d==v.d;
}

int value::operator!=(const value& v){
  if(type!=v.type)return 1;
  if(type==INTEGER)return i!=v.i;
  else return d!=v.d;
}

void value::print(FILE* f, const char* Fd, const char* Ff){
  switch(type){
    case INTEGER:
      fprintf(f,Fd,i);
      return;
    case FLOATING:
      fprintf(f,Ff,d);
      return;
  }
}

//definition of shared::shared_ops, declared externally in the header
opid shared::shared_ops[SHARED_OPS_LEN]={
  //parentheses operators
  (opid){("{}"),OP_LIST     ,0,ASOC_MULTI},   //0
  (opid){("[]"),OP_ITER1    ,0,ASOC_MULTI},   //1
  (opid){("()"),OP_ITER2    ,0,ASOC_MULTI},   //2
  //special case
  (opid){("-"),OP_MINUSSIGN ,11,ASOC_RUNARY}, //3

  (opid){("$"),OP_TAKE      ,9,ASOC_RUNARY},  //4
  (opid){("@"),OP_PUT       ,9,ASOC_RBINARY}, //5

  (opid){(":+"),OP_PPLUS    ,4,ASOC_LINFIX},  //6
  (opid){(":-"),OP_MMINUS   ,4,ASOC_LINFIX},  //7
  (opid){(":*"),OP_MMUL     ,5,ASOC_LINFIX},  //8
  (opid){(":/"),OP_DDIV     ,5,ASOC_LINFIX},  //9
  (opid){(":%"),OP_MMOD     ,5,ASOC_LINFIX},  //10
  (opid){(":^"),OP_PPOW     ,6,ASOC_RINFIX},  //11

  (opid){("@+"),OP_OPLUS    ,8,ASOC_RUNARY},  //12
  (opid){("@*"),OP_OTIMES   ,8,ASOC_RUNARY},  //13
  (opid){("avg"),OP_OMEAN   ,8,ASOC_RUNARY},  //14
  (opid){("var"),OP_OVAR    ,8,ASOC_RUNARY},  //15
  (opid){("nrm"),OP_ONORM   ,8,ASOC_RBINARY}, //16

  (opid){("!"),OP_FACT      ,9,ASOC_LUNARY},  //17
  (opid){("sin"),OP_SIN     ,8,ASOC_RUNARY},  //18
  (opid){("cos"),OP_COS     ,8,ASOC_RUNARY},  //19
  (opid){("tan"),OP_TAN     ,8,ASOC_RUNARY},  //20
  (opid){("atn"),OP_ATAN    ,8,ASOC_RUNARY},  //21
  (opid){("atan"),OP_ATAN   ,8,ASOC_RUNARY},  //22
  (opid){("asn"),OP_ASIN    ,8,ASOC_RUNARY},  //23
  (opid){("asin"),OP_ASIN   ,8,ASOC_RUNARY},  //24
  (opid){("acs"),OP_ACOS    ,8,ASOC_RUNARY},  //25
  (opid){("acos"),OP_ACOS   ,8,ASOC_RUNARY},  //26
  (opid){("abs"),OP_ABS     ,8,ASOC_RUNARY},  //27
  (opid){("exp"),OP_EXP     ,8,ASOC_RUNARY},  //28
  (opid){("sqt"),OP_SQRT    ,8,ASOC_RUNARY},  //29
  (opid){("sqrt"),OP_SQRT   ,8,ASOC_RUNARY},  //30
  (opid){("cbt"),OP_CBRT    ,8,ASOC_RUNARY},  //31
  (opid){("cbrt"),OP_CBRT   ,8,ASOC_RUNARY},  //32
  (opid){("r"),OP_ROUND     ,8,ASOC_RUNARY},  //33
  (opid){("f"),OP_FRAC      ,8,ASOC_RUNARY},  //34
  (opid){("i"),OP_INT       ,8,ASOC_RUNARY},  //35
  (opid){("d"),OP_FLOAT     ,8,ASOC_RUNARY},  //36
  (opid){("log"),OP_LOG     ,8,ASOC_RUNARY},  //37
  (opid){("ran"),OP_RAND    ,8,ASOC_RUNARY},  //38
  (opid){("bj"),OP_BESJ     ,8,ASOC_RBINARY}, //39
  (opid){("by"),OP_BESY     ,8,ASOC_RBINARY}, //40

  (opid){("+"),OP_PLUS      ,4,ASOC_LINFIX},  //41
  (opid){("-"),OP_MINUS     ,4,ASOC_LINFIX},  //42
  (opid){("*"),OP_MUL       ,5,ASOC_LINFIX},  //43
  (opid){("/"),OP_DIV       ,5,ASOC_LINFIX},  //44
  (opid){("%"),OP_MOD       ,5,ASOC_LINFIX},  //45
  (opid){("^"),OP_POW       ,6,ASOC_RINFIX},  //46
  (opid){("e"),OP_FLE       ,10,ASOC_RINFIX}, //47
  (opid){("c"),OP_NCK       ,7,ASOC_RINFIX},  //48

  (opid){("&"),OP_AND       ,3,ASOC_LINFIX},  //49
  (opid){("\\"),OP_WITHOUT  ,3,ASOC_LINFIX},  //50
  (opid){("|"),OP_OR        ,2,ASOC_LINFIX},  //51
  (opid){("|&"),OP_XOR      ,3,ASOC_LINFIX},  //52
  (opid){("~"),OP_REVERSE   ,8,ASOC_RUNARY},  //53
  (opid){("S"),OP_SORT      ,8,ASOC_RUNARY},  //54
  (opid){("#"),OP_HASH      ,8,ASOC_RUNARY},  //55
  (opid){("?"),OP_MASK      ,8,ASOC_RUNARY},  //56
  (opid){("="),OP_EQUALS    ,1,ASOC_LINFIX},  //57
  (opid){("#_"),OP_COUNT    ,8,ASOC_RBINARY}, //58

  (opid){("<<"),OP_SHL      ,7,ASOC_LINFIX},  //59
  (opid){(">>"),OP_SHR      ,7,ASOC_LINFIX},  //60
  (opid){("_"),OP_INDEX     ,7,ASOC_LINFIX},  //61
  (opid){("U"),OP_UNIQUE    ,8,ASOC_RUNARY},  //62
  (opid){("P"),OP_POSITIVE  ,8,ASOC_RUNARY},  //63
  (opid){("N"),OP_NEGATIVE  ,8,ASOC_RUNARY},  //64
  (opid){("O"),OP_ODD       ,8,ASOC_RUNARY},  //65
  (opid){("E"),OP_EVEN      ,8,ASOC_RUNARY},  //66
  (opid){("I"),OP_INTEGER   ,8,ASOC_RUNARY},  //67
  (opid){("D"),OP_DOUBLE    ,8,ASOC_RUNARY},  //68
  (opid){("min"),OP_MIN     ,8,ASOC_RUNARY},  //69
  (opid){("max"),OP_MAX     ,8,ASOC_RUNARY},  //70

  (opid){(":<"),OP_LLT      ,1,ASOC_LINFIX},  //71
  (opid){(":>"),OP_GGT      ,1,ASOC_LINFIX},  //72
  (opid){(":<="),OP_LLEQ    ,1,ASOC_LINFIX},  //73
  (opid){(":>="),OP_GGEQ    ,1,ASOC_LINFIX},  //74
  (opid){(":=="),OP_EEQ     ,1,ASOC_LINFIX},  //75
  (opid){(":<>"),OP_NNEQ    ,1,ASOC_LINFIX},  //76

  (opid){("<"),OP_LT        ,1,ASOC_LINFIX},  //77
  (opid){(">"),OP_GT        ,1,ASOC_LINFIX},  //78
  (opid){("<="),OP_LEQ      ,1,ASOC_LINFIX},  //79
  (opid){(">="),OP_GEQ      ,1,ASOC_LINFIX},  //80
  (opid){("=="),OP_EQ       ,1,ASOC_LINFIX},  //81
  (opid){("<>"),OP_NEQ      ,1,ASOC_LINFIX},  //82

  (opid){("pi"),OP_PI       ,11,ASOC_BLANK},  //83
  (opid){("aa"),OP_AA       ,11,ASOC_BLANK},  //84
  (opid){("deg"),OP_DEG     ,11,ASOC_BLANK},  //85
  (opid){("emc"),OP_EMC     ,11,ASOC_BLANK},  //86

  (opid){("pm"),OP_PM       ,11,ASOC_BLANK},  //87
  (opid){("mp"),OP_MP       ,11,ASOC_BLANK},  //88

  (opid){("++"),OP_XPLUS    ,4,ASOC_LINFIX},  //89
  (opid){("--"),OP_XMINUS   ,4,ASOC_LINFIX},  //90
  (opid){("**"),OP_XMUL     ,5,ASOC_LINFIX},  //91
  (opid){("//"),OP_XDIV     ,5,ASOC_LINFIX},  //92
  (opid){("%%"),OP_XMOD     ,5,ASOC_LINFIX},  //93
  //(opid){("^^"),OP_XPOW     ,6,ASOC_RINFIX},
  (opid){("px"),OP_PX       ,8,ASOC_RBINARY}, //94
  (opid){("pd"),OP_PD       ,8,ASOC_RUNARY},  //95
  (opid){("pz"),OP_PZ       ,8,ASOC_RUNARY},  //96

  (opid){("@:"),OP_DOT      ,5,ASOC_LINFIX},  //97
  (opid){("@="),OP_VLEN     ,8,ASOC_RUNARY},  //98
  (opid){("@!"),OP_VNRM     ,8,ASOC_RUNARY},  //99
  (opid){("@^"),OP_CROSS    ,6,ASOC_RINFIX}   //100
};

#ifdef SIGHANDLE
sig_atomic_t shared::interactive_flag=0;
sig_atomic_t shared::state=STATE_IDLE;
sig_atomic_t shared::interrupt=0; //volatile?
char* shared::wait_ptr=0;
#endif

opid findop(char* c){
  int idx=-1;
  int maxlen=0;
  int len;
  for(int i=4;i<SHARED_OPS_LEN;i++){
    len=strlen(shared::shared_ops[i].name);
    if(len<=maxlen)continue;
    if(!strncmp(shared::shared_ops[i].name,c,len)){
      maxlen=len;
      idx=i;
    }
  }
  if(idx<0)return (opid){0,0,0,0};
  return shared::shared_ops[idx];
}
