/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2011  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#include "tokener.h"

#include <climits>
#include <cctype>
#include <cstring>

using namespace std;

token token::val(value f){
  token tok;
  tok.type=TOK_VALUE;
  tok.f=f;
  return tok;
}

token token::op(opid op){
  token tok;
  tok.type=TOK_OP;
  tok.o=op;
  return tok;
}

token token::parl(uchar type){
  token tok;
  tok.type=TOK_PAR_L;
  tok.i=type;
  return tok;
}

token token::parr(uchar type){
  token tok;
  tok.type=TOK_PAR_R;
  tok.i=type;
  return tok;
}

token token::empty(){
  token tok;
  tok.type=TOK_NONE;
  return tok;
}

#if DEBUG>2
void printtoken(token& tok){
  switch(tok.type){
    case TOK_NONE: printf("none "); break;
    case TOK_PAR_L:
      switch(tok.i){
        case OP_LIST: printf("{ ");break;
        case OP_ITER1: printf("[ ");break;
        case OP_ITER2: printf("( ");break;
      }
      break;
    case TOK_PAR_R:
      switch(tok.i){
        case OP_LIST: printf("} ");break;
        case OP_ITER1: printf("] ");break;
        case OP_ITER2: printf(") ");break;
      }
      break;
    case TOK_OP:
      if(tok.o.code==OP_MINUSSIGN)
        printf("-");
      else
        printf("%s ",tok.o.name);
      break;
    case TOK_VALUE:
      tok.f.print(stdout,"%d ","%g ");
      break;
  }
}
#endif

void tokenize(char* source, vector<token>& tokens){
  uint seek=0;//string position (input is already separated into lines)
  int c=source[seek];

  int whitespace=1;

  while(c){
    #if DEBUG>2
    fprintf(stderr,"parser: pos %d, char '%c'\n",seek,c);
    #endif
    if(isspace(c)||(c==','))goto end_of_loop;
    if(c=='-'){//minus
      if(!tokens.size()){//unary minus if at the beginning
	tokens.push_back(token::op(shared::shared_ops[3]));
	goto end_of_loop;
      }
      if(whitespace&&((!isspace(source[seek+1]))&&(source[seek+1]!=',')&&(source[seek+1]!='-'))){//unary minus if preceded by space, but followed by expression
	tokens.push_back(token::op(shared::shared_ops[3]));
	goto end_of_loop;
      }
      token last=tokens[tokens.size()-1];
      //after closing brackets and operators that don't need right argument, this is unary minus
      if(((last.type==TOK_OP) && (last.o.asoc!=ASOC_LUNARY) && (last.o.asoc!=ASOC_BLANK))||last.type==TOK_PAR_L){
	tokens.push_back(token::op(shared::shared_ops[3]));
	goto end_of_loop;
      }
    }
    if(c=='+'){//plus
      if(!tokens.size())goto end_of_loop;//skip leading plus
      if(whitespace&&((!isspace(source[seek+1]))&&(source[seek+1]!=',')&&(source[seek+1]!='+')))goto end_of_loop;
      token last=tokens[tokens.size()-1];
      if(((last.type==TOK_OP) && (last.o.asoc!=ASOC_LUNARY) && (last.o.asoc!=ASOC_BLANK))||last.type==TOK_PAR_L)goto end_of_loop;
    }
    //delims
    if(c=='{'){
      tokens.push_back(token::parl(OP_LIST));
      goto end_of_loop;
    }
    if(c=='}'){
      tokens.push_back(token::parr(OP_LIST));
      goto end_of_loop;
    }
    if(c=='['){
      tokens.push_back(token::parl(OP_ITER1));
      goto end_of_loop;
    }
    if(c==']'){
      tokens.push_back(token::parr(OP_ITER1));
      goto end_of_loop;
    }
    if(c=='('){
      tokens.push_back(token::parl(OP_ITER2));
      goto end_of_loop;
    }
    if(c==')'){
      tokens.push_back(token::parr(OP_ITER2));
      goto end_of_loop;
    }
    //check for number
    if(isdigit(c)||(c=='.')){//a number encountered
      char* currentstring=source+seek;
      //this appears to be slower than manual scan (isdigit is a table lookup)
      //int numlen=strspn(currentstring,"01234567890.");
      uint numlen=1;
      while(isdigit(source[seek+numlen])||source[seek+numlen]=='.')numlen++;
      //remember what was there
      char c_temp=source[seek+numlen];
      source[seek+numlen]=0;//fake end of string
      tokens.push_back(token::val(value(currentstring)));//parse what was there
#if DEBUG>2
      fprintf(stderr,"parser: value string found: '%s'\n",currentstring);
#endif
      source[seek+numlen]=c_temp;//restore original string
      //we end up at the end of the read number
      seek+=numlen-1;
      goto end_of_loop;
    }
    {//parse operators
      opid o=findop(source+seek);
      if(o.code!=OP_NONE){
	tokens.push_back(token::op(o));
	seek+=strlen(o.name)-1;
	goto end_of_loop;
      }
    }
    //nothing matched
    fprintf(stderr,"parser: skipping bogus character '%c'\n",c);
  end_of_loop:
    whitespace=isspace(c)||(c==',');
    ++seek;
    c=source[seek];
  }
}

