/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2011  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#include "base.h"
#include "vm.h"
#include "methods.h"

#ifdef TIMERAND
#include <ctime>
#endif

#include <algorithm>
#include <cstring>
#include <cctype>

//if no history, no storing history
#ifndef HAVE_READLINE_HISTORY
#undef STOREHISTORY
#endif

#ifdef STOREHISTORY
#include<sys/stat.h>
#endif

using namespace std;


void eval_control::evaluate_value(const value& v, array& out){
  out.push_back(v);
}

array eval_control::evaluate_value(const value& v){
  array a;
  a.push_back(v);
  return a;
}


int comparator(const value v1, const value v2){
  return v1.d<v2.d;
}

//functions

#define a  args[0]
#define a1 args[0]
#define a2 args[1]

void eval_control::evaluate_operator(uchar code, vector<array>& args, array& out){
  switch(code&OPMASK){
    case OPX_MEM:{
      switch(code){
	case OP_NONE: return;
	case OP_TAKE: {
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    array* mem;
	    int index=a[i].assertI("${n}");
	    if(index<0){
	      if(-index<=history.size()){
		mem=&history[history.size()+index];
	      }else continue;
	    }else{
	      mem=&memory[index];
	    }
	    out.insert(out.end(),mem->begin(),mem->end());
	  }
	  return;
	}
	case OP_PUT:{
	  int n1=a1.size();
	  for(int i=0;i<n1;i++){
	    memory[a1[i].assertI("@{n}{...}")]=a2;
	  }
	  out=a2;
	  return;
	}
      }
      break;
    }
    case OPX_COM:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);
      switch(code){
	case OP_PPLUS:{
	  for(int i=0;i<n;i++){
	    out.push_back(a1[i]+a2[i]);
	  }
	  return;
	}
	case OP_MMINUS:{
	  for(int i=0;i<n;i++){
	    out.push_back(a1[i]-a2[i]);
	  }
	  return;
	}
	case OP_MMUL:{
	  for(int i=0;i<n;i++){
	    out.push_back(a1[i]*a2[i]);
	  }
	  return;
	}
	case OP_DDIV:{
	  for(int i=0;i<n;i++){
	    out.push_back(a1[i]/a2[i]);
	  }
	  return;
	}
	case OP_MMOD:{
	  for(int i=0;i<n;i++){
	    out.push_back(a1[i]%a2[i]);
	  }
	  return;
	}
	case OP_PPOW:{
	  for(int i=0;i<n;i++){
	    out.push_back(a1[i]^a2[i]);
	  }
	  return;
	}
      }
      break;
    }
    case OPX_ACC:{
      switch(code){
	case OP_OPLUS:{
	  int n=a.size();
	  value v=(itype)0;
	  for(int i=0;i<n;i++){
	    v=v+a[i];
	  }
	  out.push_back(v);
	  return;
	}
	case OP_OTIMES:{
	  int n=a.size();
	  value v=(itype)1;
	  for(int i=0;i<n;i++){
	    v=v*a[i];
	  }
	  out.push_back(v);
	  return;
	}
	case OP_OMEAN:{
	  int n=a.size();
	  value v=(ieee)0;
	  for(int i=0;i<n;i++){
	    v=v+a[i];
	  }
	  out.push_back(v/(itype)n);
	  return;
	}
	case OP_OVAR:{
	  int n=a.size();
	  value v=(ieee)0;
	  value v2=(ieee)0;
	  for(int i=0;i<n;i++){
	    v=v+a[i];
	    v2=v2+(a[i]*a[i]);
	  }
	  out.push_back(v2/(itype)n-((v/(itype)n)^(itype)2));
	  return;
	}
	case OP_ONORM:{
	  int n1=a1.size();
	  int n2=a2.size();
	  out.reserve(n1);
	  for(int i=0;i<n1;i++){
	    value v=(itype)0;
	    for(int j=0;j<n2;j++){
	      v=v+((a2[j].op_abs())^a1[i]);
	    }
	    out.push_back(v^(value((ieee)1)/a1[i]));
	  }
	  return;
	}
      }
      break;
    }
    case OPX_FUN1:{
      int n=a.size();
      out.reserve(n);

      switch(code){
	case OP_FACT:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_fact());
	  }
	  return;
	}
	case OP_SIN:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_sin());
	  }
	  return;
	}
	case OP_COS:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_cos());
	  }
	  return;
	}
	case OP_TAN:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_tan());
	  }
	  return;
	}
	case OP_ASIN:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_asin());
	  }
	  return;
	}
	case OP_ACOS:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_acos());
	  }
	  return;
	}
	case OP_ATAN:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_atan());
	  }
	  return;
	}
	case OP_SQRT:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_sqrt());
	  }
	  return;
	}
	case OP_CBRT:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_cbrt());
	  }
	  return;
	}
	case OP_EXP:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_exp());
	  }
	  return;
	}
	case OP_LOG:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_log());
	  }
	  return;
	}
      }
      break;
    }
    case OPX_FUN2:{
      int n=a.size();
      out.reserve(n);
      switch(code){
	case OP_MINUSSIGN:{
	  for(int i=0;i<n;i++){
	    out.push_back(-a[i]);
	  }
	  return;
	}
	case OP_ABS:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_abs());
	  }
	  return;
	}
	case OP_ROUND:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_round());
	  }
	  return;
	}
	case OP_FRAC:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_frac());
	  }
	  return;
	}
	case OP_INT:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].cast_int());
	  }
	  return;
	}
	case OP_FLOAT:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].cast_float());
	  }
	  return;
	}
	case OP_RAND:{
	  for(int i=0;i<n;i++){
	    out.push_back(a[i].op_rand());
	  }
	  return;
	}

      }
      break;
    }
    case OPX_FUNX:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);

      switch(code){
	case OP_BESJ:{
	  for(int i=0;i<n1;i++){
	    int besn=a1[i].assertI("bj{n}{...}");
	    for(int j=0;j<n2;j++){
	      out.push_back(a2[j].op_bj(besn));
	    }
	  }
	  return;
	}
	case OP_BESY:{
	  for(int i=0;i<n1;i++){
	    int besn=a1[i].assertI("by{n}{...}");
	    for(int j=0;j<n2;j++){
	      out.push_back(a2[j].op_by(besn));
	    }
	  }
	  return;
	}
      }
      break;
    }
    case OPX_TEN:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);

      switch(code){
	case OP_PLUS:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back(a1[i]+a2[j]);
	    }
	  }
	  return;
	}
	case OP_MINUS:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back(a1[i]-a2[j]);
	    }
	  }
	  return;
	}
	case OP_MUL:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back(a1[i]*a2[j]);
	    }
	  }
	  return;
	}
	case OP_DIV:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back(a1[i]/a2[j]);
	    }
	  }
	  return;
	}
	case OP_MOD:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back(a1[i]%a2[j]);
	    }
	  }
	  return;
	}
	case OP_POW:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back(a1[i]^a2[j]);
	    }
	  }
	  return;
	}
	case OP_FLE:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back(a1[i]*a2[j].op_exp10());
	    }
	  }
	  return;
	}
	case OP_NCK:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back(value::binomial(a1[i],a2[j]));
	    }
	  }
	  return;
	}

      }
      break;
    }
    case OPX_SET:{
      switch(code){
	case OP_AND:{
	  int n1=a1.size();
	  int n2=a2.size();
	  for(int i=0;i<n1;i++){
	    int b=0;
	    for(int j=0;j<n2;j++){
	      if(a1[i]==a2[j]){b=1;break;}
	    }
	    if(b)out.push_back(a1[i]);
	  }
	  return;
	}
	case OP_WITHOUT:{
	  int n1=a1.size();
	  int n2=a2.size();
	  for(int i=0;i<n1;i++){
	    int b=1;
	    for(int j=0;j<n2;j++){
	      if(a1[i]==a2[j]){b=0;break;}
	    }
	    if(b)out.push_back(a1[i]);
	  }
	  return;
	}
	case OP_OR:{
	  out.insert(out.end(),a1.begin(),a1.end());
	  out.insert(out.end(),a2.begin(),a2.end());
	  return;
	}
	case OP_XOR:{
	  int n1=a1.size();
	  int n2=a2.size();
	  for(int i=0;i<n1;i++){
	    int b=1;
	    for(int j=0;j<n2;j++){
	      if(a1[i]==a2[j]){b=0;break;}
	    }
	    if(b)out.push_back(a1[i]);
	  }
	  for(int i=0;i<n2;i++){
	    int b=1;
	    for(int j=0;j<n1;j++){
	      if(a1[j]==a2[i]){b=0;break;}
	    }
	    if(b)out.push_back(a2[i]);
	  }
	  return;
	}
	case OP_REVERSE:{
	  int n=a.size();
	  out.reserve(n);
	  for(int i=n-1;i>=0;i--){
	    out.push_back(a[i]);
	  }
	  return;
	}
	case OP_SORT:{
	  out=a;
	  sort(out.begin(),out.end(),comparator);
	  return;
	}
	case OP_HASH:{
	  int n=a.size();
	  out.push_back((itype)n);
	  return;
	}
	case OP_MASK:{
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    if(a[i]!=value((itype)0)){
	      out.push_back((itype)i);
	    }
	  }
	  return;
	}
	case OP_EQUALS:{
	  int n1=a1.size();
	  int n2=a2.size();
	  if(n1!=n2){
	    out.push_back((itype)0);
	    return;
	  }else{
	    for(int i=0;i<n1;i++){
	      if(a1[i]!=a2[i]){
		out.push_back((itype)0);
		return;
	      }
	    }
	    out.push_back((itype)1);
	    return;
	  }
	}
	case OP_COUNT:{
	  int n1=a1.size();
	  int n2=a2.size();
	  for(int i=0;i<n1;i++){
	    int count=0;
	    for(int j=0;j<n2;j++){
	      if(a1[i]==a2[j])count++;
	    }
	    out.push_back((itype)count);
	  }
	  return;
	}

      }
      break;
    }
    case OPX_SEL:{
      switch(code){
	case OP_SHL:{
	  int n1=a1.size();
	  int n2=a2.size();
	  for(int j=0;j<n2;j++){
	    int ihi=a2[j].assertI("{...}<<{n}");
	    int i1=ihi<0?-ihi:0;
	    int i2=ihi<0?n1:ihi;
	    for(int i=i1;i<i2;i++){
	      if((i<0)||(i>=n1)){out.push_back((itype)0);continue;}
	      out.push_back(a1[i]);
	    }
	  }
	  return;
	}
	case OP_SHR:{
	  int n1=a1.size();
	  int n2=a2.size();
	  for(int j=0;j<n2;j++){
	    int ihi=a2[j].assertI("{...}>>{n}");
	    int i1=ihi<0?0:n1-ihi;
	    int i2=ihi<0?n1+ihi:n1;
	    for(int i=i1;i<i2;i++){
	      if((i<0)||(i>=n1)){out.push_back((itype)0);continue;}
	      out.push_back(a1[i]);
	    }
	  }
	  return;
	}
	case OP_INDEX:{
	  int n1=a1.size();
	  if(!n1)return;
	  int n2=a2.size();

	  out.reserve(n2);
	  for(int j=0;j<n2;j++){
	    value v=a2[j]%(itype)n1;
	    if(v.type==INTEGER){
	      out.push_back(a1[v.i]);
	    }else{
	      v.fractionalize();//ensure i and f are available
	      int ii=v.i;
	      if(ii<0 || ii>=n1)continue;
	      int ii2=ii+1;
	      if(ii2>=n1)ii2-=n1;
	      out.push_back(a1[ii]*((ieee)1.-v.f)+a1[ii2]*v.f);
	    }
	  }
	  return;
	}
	case OP_UNIQUE:{
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    int b=1;
	    for(int j=0;j<out.size();j++){
	      if(out[j]==a[i]){b=0;break;}
	    }
	    if(b)out.push_back(a[i]);
	  }
	  return;
	}
	case OP_POSITIVE:{
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    if(a[i].is_positive())out.push_back(a[i]);
	  }
	  return;
	}
	case OP_NEGATIVE:{
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    if(a[i].is_negative())out.push_back(a[i]);
	  }
	  return;
	}
	case OP_ODD:{
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    if(a[i].is_odd())out.push_back(a[i]);
	  }
	  return;
	}
	case OP_EVEN:{
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    if(a[i].is_even())out.push_back(a[i]);
	  }
	  return;
	}
	case OP_INTEGER:{
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    if(a[i].is_int())out.push_back(a[i]);
	  }
	  return;
	}
	case OP_DOUBLE:{
	  int n=a.size();
	  for(int i=0;i<n;i++){
	    if(a[i].is_float())out.push_back(a[i]);
	  }
	  return;
	}
	case OP_MIN:{
	  int n=a.size();
	  if(!n)return;
	  value v=a[0];
	  for(int i=1;i<n;i++){
	    if(a[i]<v)v=a[i];
	  }
	  out.push_back(v);
	  return;
	}
	case OP_MAX:{
	  int n=a.size();
	  if(!n)return;
	  value v=a[0];
	  for(int i=1;i<n;i++){
	    if(a[i]>v)v=a[i];
	  }
	  out.push_back(v);
	  return;
	}

      }
      break;
    }
    case OPX_CLOG:{
      int n1=a1.size();
      int n2=a2.size();
      int n=n1<n2?n1:n2;
      out.reserve(n);

      switch(code){
	case OP_LLT:{
	  for(int i=0;i<n;i++){
	    out.push_back((itype)(a1[i]<a2[i]));
	  }
	  return;
	}
	case OP_GGT:{
	  for(int i=0;i<n;i++){
	    out.push_back((itype)(a1[i]>a2[i]));
	  }
	  return;
	}
	case OP_LLEQ:{
	  for(int i=0;i<n;i++){
	    out.push_back((itype)(a1[i]<=a2[i]));
	  }
	  return;
	}
	case OP_GGEQ:{
	  for(int i=0;i<n;i++){
	    out.push_back((itype)(a1[i]>=a2[i]));
	  }
	  return;
	}
	case OP_EEQ:{
	  for(int i=0;i<n;i++){
	    out.push_back((itype)(a1[i]==a2[i]));
	  }
	  return;
	}
	case OP_NNEQ:{
	  for(int i=0;i<n;i++){
	    out.push_back((itype)(a1[i]!=a2[i]));
	  }
	  return;
	}

      }
      break;
    }
    case OPX_TLOG:{
      int n1=a1.size();
      int n2=a2.size();
      out.reserve(n1*n2);
      switch(code){
	case OP_LT:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back((itype)(a1[i]<a2[j]));
	    }
	  }
	  return;
	}
	case OP_GT:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back((itype)(a1[i]>a2[j]));
	    }
	  }
	  return;
	}
	case OP_LEQ:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back((itype)(a1[i]<=a2[j]));
	    }
	  }
	  return;
	}
	case OP_GEQ:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back((itype)(a1[i]>=a2[j]));
	    }
	  }
	  return;
	}
	case OP_EQ:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back((itype)(a1[i]==a2[j]));
	    }
	  }
	  return;
	}
	case OP_NEQ:{
	  for(int i=0;i<n1;i++){
	    for(int j=0;j<n2;j++){
	      out.push_back((itype)(a1[i]!=a2[j]));
	    }
	  }
	  return;
	}

      }
      break;
    }
    case OPX_CONST:{
      switch(code){
	case OP_PI:{
	  out.push_back(PI);
	  return;
	}
	case OP_AA:{
	  out.push_back(AA);
	  return;
	}
	case OP_DEG:{
	  out.push_back(DEG);
	  return;
	}
	case OP_EMC:{
	  out.push_back(EMC);
	  return;
	}
	case OP_PM:{
	  out.push_back((itype)1);
	  out.push_back((itype)(-1));
	  return;
	}
	case OP_MP:{
	  out.push_back((itype)(-1));
	  out.push_back((itype)1);
	  return;
	}
 
      }
      break;
    }
    case OPX_POLY:{
      switch(code){
	case OP_XPLUS:{
	  int n1=a1.size();
	  int n2=a2.size();
	  int n=n1<n2?n1:n2;
	  int nx=n1>n2?n1:n2;
	  out.reserve(nx);
	  for(int i=0;i<n;i++){
	    out.push_back(a1[i]+a2[i]);
	  }
	  for(int i=n;i<nx;i++){
	    if(n1>n2){
	      out.push_back(a1[i]);
	    }else{
	      out.push_back(a2[i]);
	    }
	  }
	  return;
	}
	case OP_XMINUS:{
	  int n1=a1.size();
	  int n2=a2.size();
	  int n=n1<n2?n1:n2;
	  int nx=n1>n2?n1:n2;
	  out.reserve(nx);
	  for(int i=0;i<n;i++){
	    out.push_back(a1[i]-a2[i]);
	  }
	  for(int i=n;i<nx;i++){
	    if(n1>n2){
	      out.push_back(a1[i]);
	    }else{
	      out.push_back(-a2[i]);
	    }
	  }
	  return;
	}
	case OP_XMUL:{
	  int n1=a1.size();
	  int n2=a2.size();
	  int n=n1+n2-1;
	  out.reserve(n);
	  for(int i=0;i<n;i++){
	    value v=(itype)0;
	    for(int j=0;j<=i;j++){
	      if((j<n1)&&(i-j<n2)){
		v=v+a1[j]*a2[i-j];
	      }
	    }
	    out.push_back(v);
	  }
	  return;
	}
	case OP_XDIV:{
	  int n1=a1.size();
	  int n2=a2.size();
	  //trim the zeroes
	  for(int i=n1-1;i>=0;i--){
	    if(a1[i].d==0){
	      n1=i;
	    }else break;
	  }
	  for(int i=n2-1;i>=0;i--){
	    if(a2[i].d==0){
	      n2=i;
	    }else break;
	  }
	  if((n1==0) || (n2==0)){
	    return;
	  }
	  if(n1<n2){
	    out.push_back((itype)0);
	    return;
	  }
	  if(n1==n2){//trivial solution, when polynomials have the same order
	    out.push_back(a1[n1-1].d/a2[n2-1].d);
	    return;
	  }
	  int n=n1-n2+1;
	  out=array(a1.begin(),a1.begin()+n1);
	  for(int i=n-1;i>=0;i--){
	    if(out[i+n2-1].d==0)continue;
	    out[i+n2-1]=out[i+n2-1]/a2[n2-1].d;
	    for(int j=0;j<n2-1;j++){
	      out[i+j]=out[i+j]-a2[j]*out[i+n2-1].d;
	    }
	  }
	  out.erase(out.begin(),out.begin()+(n1-n));
	  return;
	}
	case OP_XMOD:{
	  int n1=a1.size();
	  int n2=a2.size();
	  //trim the zeroes
	  for(int i=n1-1;i>=0;i--){
	    if(a1[i].d==0){
	      n1=i;
	    }else break;
	  }
	  for(int i=n2-1;i>=0;i--){
	    if(a2[i].d==0){
	      n2=i;
	    }else break;
	  }
	  if((n1==0) || (n2==0)){
	    return;
	  }
	  if(n2==1){//always divisible
	    out.push_back((itype)0);
	    return;
	  }
	  if(n1<n2){//not divisible
	    out=a1;
	    return;
	  }
	  int n=n1-n2+1;
	  out=array(a1.begin(),a1.begin()+n1);
	  for(int i=n-1;i>=0;i--){
	    if(out[i+n2-1].d==0)continue;
	    out[i+n2-1]=out[i+n2-1]/a2[n2-1].d;
	    for(int j=0;j<n2-1;j++){
	      out[i+j]=out[i+j]-a2[j]*out[i+n2-1].d;
	    }
	  }
	  out.erase(out.begin()+(n1-n),out.end());
	  return;
	}

	case OP_PX:{
	  int n1=a1.size();
	  int n2=a2.size();
	  for(int i=0;i<n2;i++){
	    value x=a2[i];
	    if(fabs(x.d)<1){
	      value v=a1[n1-1];
	      for(int j=n1-2;j>=0;j--){
		v=v*x+a1[j];
	      }
	      out.push_back(v);
	    }else{
	      value v=a1[0];
	      for(int j=1;j<n1;j++){
		v=v/x+a1[j];
	      }
	      v=v*(x^(n2-(itype)1));
	      out.push_back(v);
	    }
	  }
	  return;
	}
	case OP_PD:{
	  int n=a.size();
	  for(int i=1;i<n;i++){
	    out.push_back(a[i]*(itype)i);
	  }
	  return;
	}
	case OP_PZ:{
	  bairstow(a,out);
	  return;
	}

      }
      break;
    }
    case OPX_VEC:{
      switch(code){
	case OP_DOT:{
	  int n1=a1.size();
	  int n2=a2.size();
	  int n=n1<n2?n1:n2;
	  value v=(itype)0;
	  for(int i=0;i<n;i++){
	    v=v+a1[i]*a2[i];
	  }
	  out.push_back(v);
	  return;
	}
	case OP_VLEN:{
	  int n=a.size();
	  value v=(itype)0;
	  for(int i=0;i<n;i++){
	    v=v+(a[i]*a[i]);
	  }
	  out.push_back(v.op_sqrt());
	  return;
	}
	case OP_VNRM:{
	  int n=a.size();
	  value v=(itype)0;
	  for(int i=0;i<n;i++){
	    v=v+(a[i]*a[i]);
	  }
	  v=v.op_sqrt();
	  for(int i=0;i<n;i++){
	    out.push_back(a[i]/v);
	  }
	  return;
	}
	case OP_CROSS:{
	  int n1=a1.size();
	  int n2=a2.size();
	  int n=n1<n2?n1:n2;
	  out.reserve(n);
	  if(n==1){out.push_back((itype)0);return;}
	  for(int i=0;i<n;i++){
	    out.push_back(a1[(i+1)%n1]*a2[(i+2)%n2]-a1[(i+2)%n1]*a2[(i+1)%n2]);
	  }
	  return;
	}

      }
      break;
    }
    case OPX_PAREN:{
      switch(code){

	case OP_LIST:{
	  for(int i=0;i<args.size();i++){
	    out.insert(out.end(),args[i].begin(),args[i].end());
	  }
	  return;
	}
	case OP_ITER1:{
	  array arg;
	  for(int i=0;i<args.size();i++){//collect
	    for(int j=0;j<args[i].size();j++){
	      arg.push_back(args[i][j]);
	    }
	  }
	  int n=arg.size();
	  switch(n){
	    case 0:{
	      out.push_back((itype)42);
	      break;
	    }
	    case 1:{
	      int count=arg[0].assertI("[n]");
	      for(int i=0;i<count;i++){
		out.push_back((itype)i);
	      }
	      break;
	    }
	    case 2:{
	      if(arg[1]>arg[0]){
		for(value v=arg[0];v<=arg[1];v=v+(itype)1){
		  out.push_back(v);
		}
	      }else{
		for(value v=arg[0];v>=arg[1];v=v-(itype)1){
		  out.push_back(v);
		}
	      }
	      break;
	    }
	    default:{
	      //same value?
	      if(arg[0].d==arg[n-1].d){
		out.push_back(arg[0]);
		break;
	      }
	      value max=(itype)0; //max. offset offset
	      value min=(itype)0;
	      value sum=(itype)0;
	      for(int i=1;i<n-1;i++){
		sum=sum+arg[i];
		max=sum>max?sum:max;
		min=sum<min?sum:min;
	      }

	      //is there minus?
	      if(arg[0]>arg[n-1]){//if the sequence is descending or if it overshot final value
		if((sum.d<0)||(min<arg[n-1]-arg[0])){
		  value v=arg[0];
		  for(int i=0;v>=arg[n-1];i++){
		    out.push_back(v);
		    v=v+arg[1+i%(n-2)];
		  }
		}
	      }else{//if the sequence is ascending or if it overshot final value
		if((sum.d>0)||(max>arg[n-1]-arg[0])){
		  value v=arg[0];
		  for(int i=0;v<=arg[n-1];i++){
		    out.push_back(v);
		    v=v+arg[1+i%(n-2)];
		  }
		}
	      }
	      break;
	    }
	  }
	  return;
	}
	case OP_ITER2:{
	  array arg;
	  for(int i=0;i<args.size();i++){//collect
	    for(int j=0;j<args[i].size();j++){
	      arg.push_back(args[i][j]);
	    }
	  }
	  int n=arg.size();
	  switch(n){
	    case 0:break;
	    case 1:{
	      out.push_back(arg[0]);
	      break;
	    }
	    case 2:{
	      for(int i=0;i<arg[1].d;i++){
		out.push_back(arg[0]);
	      }
	      break;
	    }
	    default:{
	      value v=arg[0];
	      ieee count=arg[n-1].d;
	      if(count<0)count=(-count)*(n-2)+1;//negative: whole sequence times count
	      out.reserve((int)(count+0.5));
	      for(int i=0;i<count;i++){
		out.push_back(v);
		v=v+arg[1+i%(n-2)];
	      }
	      break;
	    }
	  }
	  return;
	}

      }
      break;
    }
      //end of outer switch
  }
}

#undef a
#undef a1
#undef a2

void eval_control::evaluate(treenode& node, array& out){
  switch(node.type){
    case NODE_OP:{
      vector<array> args;
      for(list<treenode>::iterator i=node.childz.begin();i!=node.childz.end();i++){
#ifdef SIGHANDLE
        //interrupt
        if(shared::interrupt){return;}
#endif
	args.push_back(array());
        evaluate(*i,args.back());//recursion!!!
      }
      evaluate_operator(node.op.code,args,out);
      return;
    }
    case NODE_VAL:{
      evaluate_value(node.val,out);
      return;
    }
  }
}


void eval_control::print_array(array& a, FILE* stream, const char* sep, char end){
    if(!a.size()){if(end)fputc(end,stream);return;}
    for(int i=0;i<a.size()-1;i++){
      a[i].print(stream,Fd,Ff);
      if(sep)fputs(sep,stream);
#ifdef SIGHANDLE
    //interrupt - long outputs take forever, must be able to break loop
      if(shared::interrupt){fputc('\n',stream);return;}
#endif
    }
    a.back().print(stream,Fd,Ff);
    if(end)fputc(end,stream);
}

char* eval_control::strip(char* str){
  char* out=(char*)malloc(strlen(str)+1);
  char* pos=out;//current character
  char* end=out;//position of null
  char mode=0; //quote type
  for(;*str;str++){
    if(!mode){
      if(isspace(*str))continue;
      if(*str=='\''){mode='\'';continue;}
      if(*str=='"'){mode='"';continue;}
      if(*str=='\\'){mode=' ';str--;continue;}//we handle escapes later
      mode=' ';
      *pos=*str;pos++;end=pos;
      continue;
    }
    //escaped characters (not for '')
    if((*str=='\\')&&(mode!='\'')){
      switch(str[1]){
        case 0: break;
        case 't': *pos='\t';pos++;break;
        case 'n': *pos='\n';pos++;break;
        case 'r': *pos='\r';pos++;break;
        case 'f': *pos='\f';pos++;break;
        case 'b': *pos='\b';pos++;break;
        case 'v': *pos='\v';pos++;break;
        default: *pos=str[1];pos++;
      }
      //escaped characters count even at the end
      end=pos;
      if(str[1]){
        str++;
      }else{//null after escape
        fprintf(stderr,"input: escape character at the end of input\n");
      }
      continue;
    }
    if((mode!=' ')&&(*str==mode)){
      mode=' ';//reset mode
      break;
    }
    *pos=*str;pos++;
    //if no quotes, spaces at the end don't count
    if(!((mode==' ')&&isspace(*str)))end=pos;
  }
  if(!mode){//etirely empty line
    free(out);
    return 0;
  }
  if(mode!=' '){//there was no closing quote
   fprintf(stderr,"input: missing closing quote '%c'\n",mode);
  }
  *end=0;//put the null where the string ends
  return out;
}

FILE* eval_control::file_open(char* filename){
  if(!strcmp(filename,"-"))return stdin;
  FILE* f=fopen(filename,"r");
  if(f)return f;
  fprintf(stderr,"input: file '%s' could not be opened\n",filename);
  return 0;
}

array eval_control::evaluate_code(vector<code_chunk>& code, bool printlast){
  for(int i=0;i<code.size();i++){
    array a;
    evaluate(code[i].code,a);
    if(i==code.size()-1 && code[i].printable){
      if(printlast){
	print_array(a,stdout,delim,'\n');
      }
      return a;
    }
    //print if forced print is enabled or semicolon is missing
    if(code[i].printable || toggle_print)print_array(a,stdout,delim,'\n');
  }
  return array();
}


//no line comes out of precompile with newline in it!!!!
//returns the number of noncommented lines
int eval_control::precompile(char* line, char** comment,vector<code_chunk>& code){
  //convert newline into a null
  char* newline=strchr(line,'\n');
  if(newline)*newline=0;

  //shabang lines should be ignored. This way you can call ised with
  //#!/usr/local/bin/ised --f
  char* shabang=strstr(line,"#!");
  //if line begins with #!, omit it.
  //if line is empty, omit it.
  if((shabang==line)||(*line==0)){if(comment)*comment=0;return 0;}
  //if there's no comment, comment is end of string.
  if(comment)*comment=shabang?shabang:strchr(line,0);
  if(shabang)*shabang=0;//premature end of line at comment

  char* end;
  while(end=strchr(line,';')){//semicolon separates chunks of code
#ifdef SIGHANDLE
    //interrupt
    if(shared::interrupt){if(comment)*comment=0;return 0;}
#endif
    *end=0;//fake end
    vector<token> tokens;
    tokenize(line,tokens);
    compiler c;
    code.push_back((code_chunk){c.compile(tokens),false});
    *end=';';//restore end
    line=end+1;
  }
  //something remains
  if(*line){
    //finalization
    vector<token> tokens;
    tokenize(line,tokens);
    compiler c;
    code.push_back((code_chunk){c.compile(tokens),true});
    if(shabang)*shabang='#';//restore original line
    return 1;
  }else{
    if(shabang)*shabang='#';//restore original line
    return 0;
  }
}

char* eval_control::formatdup(const char* fmt){
#if IVALUE==ILONG || VALUE==LONG
  int len=0;
  int percent=0;
  for(const char* c=fmt;*c;c++){
    len++;
    if(*c == '%'){
      percent=1;
    }else{
      if(percent && isalpha(*c)){//percent is closed by a specifier
	switch(*c){
	  case '%':
	    break;
	  //already long specifier, no problem
	  //if shouldn't be there, not our problem
	  case 'l':
	  case 'L':
	    break;
	  //percent + integer specifier = extension to long needed
#if IVALUE==ILONG
	  case 'd':
	  case 'x':
	  case 'X':
	  case 'i':
	  case 'u':
	    len++;
	    break;
#endif
#if VALUE==LONG
	  case 'e':
	  case 'E':
	  case 'f':
	  case 'g':
	  case 'G':
	    len++;
	    break;
#endif
	}	      
	percent=0;//percent + alphanumeric means percent closed
      }
    }
  }
  //output string is allocated
  char* out=(char*)malloc(len+1);
  char* out_chr=out;
  percent=0;
  //now for real
  for(const char* c=fmt;*c;c++){
    if(*c == '%'){
      percent=1;
    }else{
      if(percent && isalpha(*c)){//percent is closed by a specifier
	switch(*c){
	  case '%':
	    break;
	  //already long specifier, no problem
	  //if shouldn't be there, not our problem
	  case 'l':
	  case 'L':
	    break;
	  //percent + integer specifier = extension to long needed
#if IVALUE==ILONG
	  case 'd':
	  case 'x':
	  case 'X':
	  case 'i':
	  case 'u':
	    *out_chr='l';
	    out_chr++;
	    len++;
	    break;
#endif
#if VALUE==LONG
	  case 'e':
	  case 'E':
	  case 'f':
	  case 'g':
	  case 'G':
	    *out_chr='L';
	    out_chr++;
	    len++;
	    break;
#endif
	}	      
	percent=0;//percent + alphanumeric means percent closed
      }
    }
    *out_chr=*c;
    out_chr++;
  }
  *out_chr=0;//null terminated string
  return out;
#else
  return strdup(fmt);
#endif
}

char* eval_control::bufgets(FILE* file){
   char* out=0;
   //put some dummy non-null character
   buffer[bufsize-1]=' ';
   out=fgets(buffer,bufsize,file);
   //if it changed, the buffer was too short
   while(buffer[bufsize-1]!=' '){
     #if DEBUG>0
     fprintf(stderr,"input: buffer resize from %d to %d\n",bufsize,2*bufsize);
     #endif
     buffer=(char*)realloc(buffer,2*bufsize);
     buffer[2*bufsize-1]=' ';
     out=fgets(buffer+bufsize-1,bufsize+1,file);
     bufsize*=2;
   }
   return out;
}

#ifndef HAVE_LIBREADLINE
char* readline(const char* prompt){
  fputs(prompt,stdout);
  char* buf=(char*)malloc(BUF_SIZE);
  return fgets(buf,BUF_SIZE,stdin);
}
#endif


void eval_control::read_interactive(){
  credits_dump();
#ifdef STOREHISTORY
  char* home=getenv("HOME");
  char* hist_file=0;
  if(home){
    hist_file=(char*)malloc(strlen(home)+15);
    sprintf(hist_file,"%s/.ised_history",home);

    struct stat buf;
    if(!stat(hist_file,&buf)){//file exists
      if(int errno=read_history(hist_file)){
	fprintf(stderr,"readline: errno %d on history load\n",errno);
      }
    }
  }
#endif

#ifdef SIGHANDLE
  shared::wait_ptr=&awaiting;
#endif
  char prompt[7];
  strcpy(prompt,"ised> ");
  char* line;
  while(line=readline(prompt)){
#ifdef SIGHANDLE
    shared::state=STATE_WORK;
#endif
    //check for interactive interrupt
    if(!strcmp(line,"q")|| !strcmp(line,"quit"))break;
    input_line(line);
#ifdef SIGHANDLE
    shared::state=STATE_IDLE;
    shared::interrupt=0;
#endif
#ifdef HAVE_READLINE_HISTORY
    if(strlen(line))add_history(line);
#endif
    free(line);
    //change prompt:
    if(awaiting){
      sprintf(prompt,"%c= ",awaiting);
    }else{
      strcpy(prompt,"ised> ");
    }
  }
#ifdef SIGHANDLE
  shared::wait_ptr=0;
#endif

#ifdef STOREHISTORY
  if(home){
    if(int errno=write_history(hist_file)){
      fprintf(stderr,"readline: errno %d on history write\n",errno);
    }else{
      history_truncate_file(hist_file,256);
    }
  }
  if(hist_file)free(hist_file);
#endif

}



void eval_control::read_file(FILE* f){
  while(bufgets(f)){
    input_line(buffer);
  }
}


array eval_control::evaluate_line(char* line, char** comment, bool printlast){
  vector<code_chunk> code;
  precompile(line,comment,code);
  return evaluate_code(code,printlast);
}

void eval_control::load_file(FILE* f){
  char* comment;
  array a;
  while(bufgets(f)){
    a=evaluate_line(buffer,&comment,false);
    //save to memory unless it's commented or empty line
    if(comment)memory[++mempos]=a;
  }
  memory[0]=evaluate_value((itype)mempos);
}

//returns 0 for normal code, 1 for nonprintable code and 2 for immediate return
int  eval_control::retrieve_code(char* f, vector<code_chunk>& code, bool file_or_line){
  char* comment;
  int outputlines=0;
  if(file_or_line){
    FILE* ffile=file_open(f);
    if(!ffile)return 2;
    while(bufgets(ffile)){
      outputlines=precompile(buffer,&comment,code);
    }

    fclose(ffile);
  }else{
    outputlines=precompile(f,&comment,code);
    if(!comment)return 2;//empty, failed or entirely commented
  }
  return outputlines==0?1:0;
}

void eval_control::apply_to_file(char* f, FILE* l, bool file_or_line){
  char* comment;

  vector<code_chunk> code;
  int errcode=retrieve_code(f,code,file_or_line);
  if(errcode==2)return;
  int line=1;
  while(bufgets(l)){
    memory[0]=evaluate_value((itype)line);
    memory[1]=evaluate_line(buffer,&comment,false);
    if(comment)line++;
    if(comment){
      array a=evaluate_code(code,false);
      if(!errcode){//only print if there's something to print
	history.push_back(a);
        print_array(a,stdout,delim,0);
        if(*comment){
	  fputs(comment,stdout);
        }
	fputc('\n',stdout);
      }
    }else{//if empty, failed or entirely commented, preserve it (preserves blanks)
      if(!errcode){
	fputs(buffer,stdout);
	fputc('\n',stdout);
      }
    }
  }
}

void eval_control::interlace_code(vector<code_chunk>& code){
  vector<array> temp_buffer;
  for(int i=0;i<code.size();i++){
    array a;
    evaluate(code[i].code,a);
    temp_buffer.push_back(a);
  }

  int index=0;
  while(1){
    array a;
    for(int j=0;j<temp_buffer.size();j++){
      if(temp_buffer[j].size()>index){
        a.push_back(temp_buffer[j][index]);
      }
    }
    if(a.size()>0){
      print_array(a,stdout,delim,'\n');
      index++;
    }else break;
  }
}

void eval_control::interlace_to_line(char* line){
  vector<code_chunk> code;
  precompile(line,0,code);
  interlace_code(code);
}


void eval_control::interlace_to_file(char* f, FILE* l){
  vector<code_chunk> code;
  precompile(f,0,code);

  char* comment;

  int line=1;
  while(bufgets(l)){
    memory[0]=evaluate_value((itype)line);
    memory[1]=evaluate_line(buffer,&comment,false);
    if(comment){
      line++;
      interlace_code(code);
    }else{
      fputs(buffer,stdout);
    }
  }
}

void eval_control::memory_dump(){
  typedef map<int,array>::iterator iterator;
  fprintf(stderr,"memory page:\n");
  for(iterator i=memory.begin();i!=memory.end();i++){
    fprintf(stderr,"@%+6d | ",(*i).first);
    array a=(*i).second;
    print_array(a,stderr," ",'\n');
  }
  fprintf(stderr,"end of memory page\n");
}

void eval_control::credits_dump(){
fprintf(stderr,"%s  Copyright (C) 2009-2011 Simon Copar\n\
This program comes with ABSOLUTELY NO WARRANTY.\n\
This is free software, and you are welcome to redistribute it\n\
under certain conditions.\n",
PACKAGE_STRING);
}

static const char* STR_DISABLED="DISABLED";
static const char* STR_ENABLED="ENABLED";

void eval_control::configure_dump(){
#if VALUE==FLOAT
const char* dstr="FLOAT";
#elif VALUE==DOUBLE
const char* dstr="DOUBLE";
#elif VALUE==LONG
const char* dstr="LONG DOUBLE";
#endif

#if IVALUE==IINT
const char* istr="INT";
#elif IVALUE==ILONG
const char* istr="LONG LONG";
#endif
fprintf(stderr,"Build parameters:\n\
  floating point precision=%s (%d bytes)\n\
  integer precision=%s (%d bytes)\n",
  dstr,sizeof(ieee),istr,sizeof(itype));
#ifndef HAVE_LIBREADLINE
 const char* lrlstr=STR_DISABLED;
#else
 const char* lrlstr=STR_ENABLED;
#endif
#ifndef HAVE_READLINE_HISTORY
 const char* rlhstr=STR_DISABLED;
#else
 const char* rlhstr=STR_ENABLED;
#endif
#ifndef STOREHISTORY
 const char* rlhstore=STR_DISABLED;
#else
 const char* rlhstore=STR_ENABLED;
#endif
#ifndef TRAP_OVR
 const char* ovrflw=STR_DISABLED;
#else
 const char* ovrflw=STR_ENABLED;
#endif
 fprintf(stderr,"  libreadline %s, history %s, history store %s, overflow handling %s\n", lrlstr,rlhstr,rlhstore,ovrflw);

}

void eval_control::input_line(char* line){
  char* tofree=0;
  if(!awaiting){
    if(!strncmp(line,"--m",3)){memory_dump();return;}
    if(!strncmp(line,"--p",3)){toggle_print=1;return;}
    if(!strncmp(line,"--n",3)){toggle_print=0;return;}
    if(!strncmp(line,"--v",3)){
      credits_dump();
      putc('\n',stderr);
      configure_dump();
      return;
    }
    const char* chars="faldtDFx";
    char compare[4]="---";

    while(*chars){
      compare[2]=*chars;
      if(!strncmp(line,compare,3)){
	tofree=line=strip(line+3);
        awaiting=*chars;
        break;
      }
      chars++;
    }
  }

  //waiting for something
  if(awaiting){
    if((!tofree)&&line){
      if(awaiting=='t'){
	tofree=line=strdup(line);
      }else{
	tofree=line=strip(line);
      }
    }
    if(!line)return;//really empty line
    switch(awaiting){
      case 'f':{
        awaiting=0;
	if(lfile){//if we want to apply to another file
	  FILE* l=file_open(lfile);
	  if(l){
	    if(xltoggle==XL_LINE){
	      apply_to_file(line,l,true);
	    }else{
	      extended_to_file(line,l,true);
	    }
	    fclose(l);
	  }
	  free(lfile);
	  lfile=0;
	}else{
	  FILE* f=file_open(line);
	  if(f){
	    read_file(f);//just read
	    fclose(f);
	  }
	}
        break;
      }
      case 'a':{
        awaiting=0;
        FILE* f=file_open(line);
        if(f){
          load_file(f);//load into memory
          fclose(f);
        }
        break;
      }
      case 'l':{
        awaiting=0;
	xltoggle=XL_LINE;
        if(lfile)free(lfile);
        lfile=strdup(line);
        break;
      }
      case 'x':{
	awaiting=0;
	xltoggle=XL_EXTENDED;
	if(lfile)free(lfile);
	lfile=strdup(line);
	break;
      }
      case 'd':{
        awaiting=0;
        delim=strdup(line);
        break;
      }
      case 't':{
        awaiting=0;
        if(lfile){//if we want to apply to another file
            FILE* l=file_open(lfile);
            if(l){
                interlace_to_file(line,l);
                fclose(l);
            }
            free(lfile);
            lfile=0;
        }
        else interlace_to_line(line);
        break;
      }
      case 'D':{
        awaiting=0;
        free(Fd);
        Fd=formatdup(line);
        break;
      }
      case 'F':{
        awaiting=0;
        free(Ff);
        Ff=formatdup(line);
        break;
      }
    }
    awaiting=0;
    if(tofree){
      free(tofree);
      tofree=0;
    }
    return;
  }

  //empty line?
  if(!*line)return;

  //ordinary line of code
  if(lfile){//applying to a file?
    FILE* l=file_open(lfile);
    if(l){
      if(xltoggle==XL_LINE){
	apply_to_file(line,l,false);
      }else{
	extended_to_file(line,l,false);
      }
      fclose(l);
    }
    free(lfile);
    lfile=0;
  }else{//just a line
    char* comment;
    array a=evaluate_line(line,&comment,true);
    if(comment){
      history.push_back(a);
    }
  }
}

int eval_control::format_splitline(char* line, vector<format_piece>& fmt){
  int countnums=0;
  char* c=line;

  format_piece p=(format_piece){'s',c,c};
  const char* delims=" \t,;:";
  while(*c && *c!='\n'){//iteration
    if(p.c=='d'){//number
      char* end;
      strtold(c,&end);//find the end of the number
      p.parsed_start=c;//this 'd' part of the condition only happens at the beginning of the number
      p.parsed_stop=end;
      if(end-c>0){
	fmt.push_back(p);//write down the number
	countnums++;
	p=(format_piece){'s',end,end};//new string
	c=end;
	continue;
      }else{
	p.c='s';
	c=end;
      }
    }else{//string
      if(isdigit(*c)||*c=='.'||*c=='-'){//found number, minus may start a number
	if(c==p.parsed_start){//a new number immediately, convert to number (this happens at the beginning)
	  p.c='d';
	  continue;
	}else{
	  if(strchr(delims,*(c-1)) ){//if previous is a delimiter
	    p.parsed_stop=c;
	    fmt.push_back(p);
	    p=(format_piece){'d',c,c};//new number
	    continue;
	  }else{//if not after delimiter, part of a string
	    //do nothing
	  }
	}
      }
    }
    c++;
  }
  //cleanup: if anything is still in the "p", add to format
  if(c>p.parsed_start){
    p.parsed_stop=c;
    fmt.push_back(p);
    if(p.c=='d')countnums++;
  }
  return countnums;
}

void eval_control::extended_to_file(char* l, FILE* f, bool file_or_line){
  vector<format_piece> fmt;

  //compile code
  vector<code_chunk> code;
  int errcode=retrieve_code(l,code,file_or_line);
  if(errcode==2)return;

  int line=1;
  while(bufgets(f)){
    memory[0]=evaluate_value((itype)line);
    //parse. If no numbers, just return the same number and carry on
    fmt.clear();
    if(!format_splitline(buffer,fmt)){
      fputs(buffer,stdout);
      continue;
    }
    //here we fill the memory with slots from the input line
    memory[1].clear();
    for(int i=0;i<fmt.size();i++){
      format_piece fp=fmt[i];
      if(fp.c=='d'){//number
	char end=*fp.parsed_stop;
	*fp.parsed_stop=0;//fake end of string
	memory[1].push_back(value(fp.parsed_start));
	*fp.parsed_stop=end;//restore original character
      }
    }
    //evaluate the line
    array a=evaluate_code(code,false);
    line++;
    if(!errcode){
      int outsize=a.size();
      int outpos=0;
      for(int i=0;i<fmt.size();i++){
	format_piece fp=fmt[i];
	if(fp.c=='d'){//number
	  if(outpos<outsize){
	    a[outpos].print(stdout,Fd,Ff);
	    outpos++;
	  }
	}else{//string
	  char end=*fp.parsed_stop;
	  *fp.parsed_stop=0;//fake end of string
	  fputs(fp.parsed_start,stdout);
	  *fp.parsed_stop=end;//restore original character
	}
      }
      //if there's anything left
      while(outpos<outsize){
	fputs(delim,stdout);
	a[outpos].print(stdout,Fd,Ff);
	outpos++;
      }
      putc('\n',stdout);//terminate line
    }
  }

}


eval_control::eval_control(){//initialization
#ifdef TIMERAND
  srand(time(0));
#else
  {
    uint seed;
    FILE* f=fopen("/dev/urandom","r");
    fread(&seed,sizeof(uint),1,f);
    srand(seed);
    fclose(f);
  }
#endif
  toggle_print=0;
  mempos=0;
  awaiting=0;
  lfile=0;
  xltoggle=XL_LINE;
  delim=strdup(" ");
  Fd=formatdup("%d");
  Ff=formatdup("%g");
  bufsize=BUF_SIZE;
  buffer=(char*)malloc(bufsize);
}

eval_control::~eval_control(){
  //free buffer
  free(buffer);
  //free other allocated strings
  free(delim);
  free(Fd);
  free(Ff);
}
